#include "bintree.h"

/**************************************
 *        TREE NODE ALLOCATION
 *************************************/

bintreenode_t* malloc_bintreenode(data_t data)
{
    bintreenode_t *node = (bintreenode_t*)malloc(sizeof(bintreenode_t));
    if (node == NULL) {
        fprintf(stderr, "ERROR: Cannot allocate memory for link list node.");
        exit(EXIT_FAILURE);
    }
    node->data = data;
    node->left = NULL;
    node->right = NULL;
    return node;
}


/**************************************
 *         PROVIDED FUNCTIONS
 *************************************/

void bintree_init(bintree_t *tree)
{
    tree->root = NULL;
}

/*
 * Helper for bintree_size.
 */
size_t bintreenode_size(bintreenode_t *node)
{
    if (node == NULL) {
        return 0;
    } else {
        return 1 + bintreenode_size(node->left) + bintreenode_size(node->right);
    }
}

size_t bintree_size(bintree_t *tree)
{
    return bintreenode_size(tree->root);
}

/*
 * Helper for bintree_height.
 */
size_t bintreenode_height(bintreenode_t *node)
{
    if (node == NULL) {
        // empty tree
        return 0;
    } else if (node->left == NULL && node->right == NULL) {
        // leaf node
        return 0;
    } else if (node->left == NULL) {
        // single child (right)
        return 1 + bintreenode_height(node->right);
    } else if (node->right == NULL) {
        // single child (left)
        return 1 + bintreenode_height(node->left);
    } else {
        // two children
        return 1 + MAX(bintreenode_height(node->left),
                       bintreenode_height(node->right));
    }
}

size_t bintree_height (bintree_t *tree)
{
    return bintreenode_height(tree->root);
}


/**************************************
 *         TO BE IMPLEMENTED
 *************************************/

void bintree_preorder(bintree_t *tree)
{
}

void bintree_postorder(bintree_t *tree)
{
}

void bintree_inorder(bintree_t *tree)
{
}

void bintree_breadth_first(bintree_t *tree)
{
}

void bintree_random_add(bintree_t *tree, data_t value)
{
}

void bintree_free(bintree_t *tree)
{
}

